@@ -14,12 +14,10 @@ pub fn crate_winmds() -> &'static [File] {
     unsafe { &*VALUE.as_ptr() }
 }
 
-pub fn workspace_dir() -> std::path::PathBuf {
+fn cargo_metadata() -> &'static str {
     use std::{mem::MaybeUninit, sync::Once};
     static ONCE: Once = Once::new();
-    static mut VALUE: MaybeUninit<std::path::PathBuf> = MaybeUninit::uninit();
-
-    // TODO: calling `cargo metadata` just to get the workspace dir takes about 40ms.
+    static mut VALUE: MaybeUninit<String> = MaybeUninit::uninit();
 
     ONCE.call_once(|| {
         let output = std::process::Command::new(env!("CARGO"))
@@ -30,31 +28,63 @@ pub fn workspace_dir() -> std::path::PathBuf {
             .output()
             .expect("Failed to run `cargo metadata`");
 
-        const JSON_KEY: &str = r#""workspace_root":"#;
-        let json = String::from_utf8(output.stdout).expect("Cargo metadata is not utf-8");
-        let beginning_index = json
-            .rfind(JSON_KEY)
-            .expect("Cargo metadata did not contain `workspace_root` key.")
-            + JSON_KEY.len()
-            + 1;
-
-        let ending_index = json[beginning_index..]
-            .find('"')
-            .expect("Cargo metadata ended before closing '\"' in `workspace_root` value");
-
-        let workspace_root =
-            json[beginning_index..beginning_index + ending_index].replace("\\\\", "\\");
-
-        // This is safe because `Once` provides thread-safe one-time initialization
-        unsafe { VALUE = MaybeUninit::new(workspace_root.into()) }
+        unsafe {
+            VALUE = MaybeUninit::new(
+                String::from_utf8(output.stdout).expect("Cargo metadata is not utf-8"),
+            )
+        }
     });
 
     // This is safe because `call_once` has already been called.
-    unsafe { (*VALUE.as_ptr()).clone() }
+    unsafe { &*VALUE.as_ptr() }
+}
+
+pub fn workspace_dir() -> String {
+    const JSON_KEY: &str = r#""workspace_root":"#;
+    let json = cargo_metadata();
+
+    let beginning_index = json
+        .rfind(JSON_KEY)
+        .expect("Cargo metadata did not contain `workspace_root` key.")
+        + JSON_KEY.len()
+        + 1;
+
+    let ending_index = json[beginning_index..]
+        .find('"')
+        .expect("Cargo metadata ended before closing '\"' in `workspace_root` value");
+
+    json[beginning_index..beginning_index + ending_index].replace("\\\\", "\\")
+}
+
+pub fn target_dir() -> String {
+    const JSON_KEY: &str = r#""target_directory":"#;
+    let json = cargo_metadata();
+
+    let beginning_index = json
+        .rfind(JSON_KEY)
+        .expect("Cargo metadata did not contain `target_directory` key.")
+        + JSON_KEY.len()
+        + 1;
+
+    let ending_index = json[beginning_index..]
+        .find('"')
+        .expect("Cargo metadata ended before closing '\"' in `target_directory` value");
+
+    json[beginning_index..beginning_index + ending_index].replace("\\\\", "\\")
 }
 
 fn get_crate_winmds() -> Vec<File> {
     fn push_dir(result: &mut Vec<File>, dir: &std::path::PathBuf) {
+        {
+            if let Ok(mut file) = std::fs::OpenOptions::new()
+                .append(true)
+                .create(true)
+                .open("C:\\git\\test_target_dir.txt")
+            {
+                use std::io::prelude::*;
+                let _ = writeln!(file, "looking in {}", target_dir());
+            }
+        }
         if let Ok(files) = std::fs::read_dir(&dir) {
             for file in files.filter_map(|file| file.ok()) {
                 if let Ok(file_type) = file.file_type() {
@@ -81,17 +111,7 @@ fn get_crate_winmds() -> Vec<File> {
     dir.push("winmd");
     push_dir(&mut result, &dir);
 
-    let dir = std::env::var("PATH").expect("No `PATH` env variable set");
-    let end = dir.find(';').expect("Path not ending in `;`");
-    let mut dir: std::path::PathBuf = dir[..end].into();
-    dir.pop();
-    dir.pop();
-    dir.push(".windows");
-    dir.push("winmd");
-    push_dir(&mut result, &dir);
-
-    let mut dir = workspace_dir();
-    dir.push("target");
+    let mut dir: std::path::PathBuf = target_dir().into();
     dir.push(".windows");
     dir.push("winmd");
     push_dir(&mut result, &dir);
